<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Observer;

use Magento\Framework\Event\ObserverInterface;
use FiloBlu\ExtAmastyGeoipRedirect\Model\Source\Logic;
use Magento\Store\Model\ScopeInterface;

class Redirect implements ObserverInterface {

    protected $_helper;
    protected $redirectAllowed = false;

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Amasty\GeoipRedirect\Helper\Data
     */
    protected $geoipHelper;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Amasty\Geoip\Model\Geolocation
     */
    protected $geolocation;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Store\Api\StoreCookieManagerInterface
     */
    protected $storeCookieManager;
    protected $_multiDomainHelper;

    public function __construct(
    \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress, \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, \Amasty\GeoipRedirect\Helper\Data $geoipHelper, \Magento\Framework\UrlInterface $urlBuilder, \Magento\Store\Model\StoreManagerInterface $storeManager, \Amasty\Geoip\Model\Geolocation $geolocation, \Magento\Customer\Model\Session $customerSession, \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager, \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper, \FiloBlu\ExtAmastyGeoipRedirect\Model\Logger\Logger $logger, \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper
    ) {
        $this->_helper = $helper;
        $this->_logger = $logger;
        $this->remoteAddress = $remoteAddress;
        $this->scopeConfig = $scopeConfig;
        $this->geoipHelper = $geoipHelper;
        $this->urlBuilder = $urlBuilder;
        $this->storeManager = $storeManager;
        $this->geolocation = $geolocation;
        $this->customerSession = $customerSession;
        $this->storeCookieManager = $storeCookieManager;
        $this->_multiDomainHelper = $multiDomainHelper;
    }

    public function execute(\Magento\Framework\Event\Observer $observer) {

        $request = $observer->getRequest();
        $response = $observer->getResponse();


        if ($request->isXmlHttpRequest()) {
            return;
        }


        $currentStoreId = $this->storeManager->getStore()->getId();
        $scopeStores = ScopeInterface::SCOPE_STORES;

        $logEnabled = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log', $scopeStores, $currentStoreId);
        $logEnabledForCookie = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_cookie', $scopeStores, $currentStoreId);
        $logEnabledForIps = $this->scopeConfig->getValue('amgeoipredirect/debug/enable_log_for_ip', $scopeStores, $currentStoreId);

        $ipRestriction = $this->scopeConfig->getValue('amgeoipredirect/restriction/ip_restriction');
        $currentIp = $this->remoteAddress->getRemoteAddress();
        $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
        $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

        if ($logEnabled && $logEnabledForIps && !$currentIp) {
            $this->_logger->info("Can't get ip");
        }

        if ($logEnabled && $logEnabledForIps) {
            if (strpos($logEnabledForIps, $currentIp) != false || $logEnabledForIps == $currentIp) {
                
            } else {
                $logEnabled = false;
            }
        }

        if ($logEnabled && $logEnabledForCookie && !$this->_helper->getCanLogCookie()) {
            $logEnabled = false;
        }

        if ($logEnabled) {
            $this->_logger->info(" #### START REQUEST #### " . $this->urlBuilder->getCurrentUrl() . " controller->" . $request->getControllerModule());
        }


        if ($logEnabled) {
            $this->_logger->info("Current Ip: " . $currentIp);

            if ($fakeIp) {
                $this->_logger->info("Fake Ip: " . $fakeIp);
            }

            if ($fakeCountry) {
                $this->_logger->info("Fake Country: " . $fakeCountry);
            }
        }

        if ($fakeIp) {
            $currentIp = $fakeIp;
        }


        if (!empty($ipRestriction)) {
            $ipRestriction = array_map("rtrim", explode(PHP_EOL, $ipRestriction));
            foreach ($ipRestriction as $ip) {
                if ($currentIp && $currentIp == $ip) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Ip restriction " . $ip);
                    }
                    return;
                }
            }
        }
        $isApi = $request->getControllerModule() == 'Mage_Api';
        if ($isApi || !$this->geoipHelper->isModuleOutputEnabled('Amasty_Geoip')) {
            if ($logEnabled) {
                $this->_logger->info("No Redirect For No Output enabled Module or API Call");
            }
            return;
        }
        $userAgent = $request->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map("trim", $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    if ($logEnabled) {
                        $this->_logger->info("No Redirect For Not Allowed User agent " . $userAgent);
                    }
                    return;
                }
            }
        }

        $this->applyLogic($request, $logEnabled);

        if ($this->redirectAllowed && !$this->geolocationRequest($request)) {
            $head = '<script>window.filoblu_geoip_redirect_allowed = true;</script>';
            $response->setBody(preg_replace('/<\/head>/', $head . '</head>', $response->getBody(), 1));
        }

        if ($logEnabled) {
            if ($this->redirectAllowed) {
                $this->_logger->info("redirect is allowed");
            } else {
                $this->_logger->info("redirect is not allowed");
            }
        }

        if ($this->scopeConfig->getValue('amgeoipredirect/general/enable', $scopeStores, $currentStoreId) && $this->redirectAllowed) {

            $location = $this->geolocation->locate($currentIp);
            $country = $location->getCountry();

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            if ($logEnabled) {
                $this->_logger->info("Current Country: " . $country);
            }

            if ($country) {
                $this->_helper->setLocalizedCountryCookie($country);
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/restriction/first_visit_redirect')) {
                $getAmYetRedirectStore = $this->_helper->getStoreCookie();
                $getAmYetRedirectCurrency = $this->_helper->getCurrencyCookie();
                $getAmYetRedirectUrl = $this->_helper->getUrlCookie();
            } else {
                $getAmYetRedirectStore = 0;
                $getAmYetRedirectCurrency = 0;
                $getAmYetRedirectUrl = 0;
            }

            if (!$getAmYetRedirectUrl && $this->scopeConfig->getValue('amgeoipredirect/country_url/enable_url')) {
                $urlMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_url/url_mapping', 'default', $currentStoreId));
                $currentUrl = $this->urlBuilder->getCurrentUrl();
                foreach ($urlMapping as $countries => $url) {
                    if (strpos($countries, $country) !== false && $url != $currentUrl) {
                        $this->_helper->setUrlCookie();
                        $response->setRedirect($url);
                        $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
                        $response->setHeader('Pragma', 'no-cache', true);
                        $response->sendResponse();
                        exit;
                    }
                }
            }

            if ($this->scopeConfig->getValue('amgeoipredirect/country_store/enable_store')) {

                $redirect_always_from_slash = $this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_always_from_slash');
                $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));



                if ($getAmYetRedirectStore) {

                    $can_redirect = true;

                    if ($getAmYetRedirectStore == $this->storeManager->getStore()->getId()) {
                        $can_redirect = false;
                    }

                    if ($redirect_always_from_slash && $currentPath == "/") {
                        $can_redirect = true;
                    }

                    if ($can_redirect) {

                        $currentStoreUrl = str_replace('&amp;', '&', $this->storeManager->getStore($getAmYetRedirectStore)->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl, '/');

                        if ($logEnabled) {
                            $this->_logger->info("Cookie is set, redirect to ->" . $redirectStoreUrl);
                        }

                        $this->storeCookieManager->setStoreCookie($this->storeManager->getStore($getAmYetRedirectStore));
                        $response->setRedirect($redirectStoreUrl);
                        $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
                        $response->setHeader('Pragma', 'no-cache', true);
                        $response->sendResponse();
                    } else {
                        if ($logEnabled) {
                            $this->_logger->info("Cookie is set, store is correct , no redirect required");
                        }
                    }
                } else {


                    if ($country) {
                        $this->_helper->setCountryCookie($country);

                        if ($logEnabled) {
                            $this->_logger->info("Set country cookie->" . $country);
                        }
                    } else {
                        $countryid = $this->scopeConfig->getValue('general/country/default', $scopeStores);
                        $this->_helper->setCountryCookie($countryid);
                        if ($logEnabled) {
                            $this->_logger->info("Set default country cookie->" . $country);
                        }
                    }

                    $allStores = $this->storeManager->getStores();

                    $currentGroup = $this->_multiDomainHelper->getCurrentGroup();

                    foreach ($allStores as $store) {

                        if ($currentGroup && $currentGroup != $this->_multiDomainHelper->getCurrentGroup($store->getWebsite()->getCode())) {
                            continue;
                        }

                        $currentStoreUrl = str_replace('&amp;', '&', $store->getCurrentUrl(false));
                        $redirectStoreUrl = trim($currentStoreUrl, '/');
                        $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());
                        if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_between_websites')) {
                            $useMultistores = $store->getWebsiteId() == $this->storeManager->getStore()->getWebsiteId();
                        } else {
                            $useMultistores = true;
                        }

                        $can_redirect = true;

                        if ($store->getId() == $currentStoreId) {
                            $can_redirect = false;
                        }

                        if ($redirect_always_from_slash && $currentPath == "/") {
                            $can_redirect = true;
                        }

                        if ($country && $countries && strpos($countries, $country) !== false && $can_redirect && $useMultistores
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Geolocalized, redirect to " . $redirectStoreUrl);
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            $response->setRedirect($redirectStoreUrl);
                            $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
                            $response->setHeader('Pragma', 'no-cache', true);
                            $response->sendResponse();
                            exit;
                        }

                        if ($country && $countries && strpos($countries, $country) !== false && $store->getId() == $currentStoreId
                        ) {
                            if ($logEnabled) {
                                $this->_logger->info("Already in the right website, set only cookie no redirect necessary");
                            }

                            $this->_helper->setStoreCookie($store->getId());
                            $this->storeCookieManager->setStoreCookie($store);
                            break;
                        }
                    }
                }

                $response->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0', true);
                $response->setHeader('Pragma', 'no-cache', true);
            }

            if (!$getAmYetRedirectCurrency && $this->scopeConfig->getValue('amgeoipredirect/country_currency/enable_currency')) {
                $currencyMapping = unserialize($this->scopeConfig->getValue('amgeoipredirect/country_currency/currency_mapping', $scopeStores, $currentStoreId));
                foreach ($currencyMapping as $countries => $currency) {
                    if (strpos($countries, $country) !== false && $this->storeManager->getStore()->getCurrentCurrencyCode() != $currency) {
                        $this->_helper->setCurrencyCookie();
                        $this->storeManager->getStore()->setCurrentCurrencyCode($currency);
                    }
                }
            }
        }

        if ($logEnabled) {
            $this->_logger->info(" #### END REQUEST #### ");
        }
    }

    protected function applyLogic($request, $logEnabled) {

        $applyLogic = $this->scopeConfig->getValue('amgeoipredirect/restriction/apply_logic');
        $currentUrl = $this->urlBuilder->getCurrentUrl();
        $baseUrl = $this->storeManager->getStore()->getCurrentUrl();

        if ($this->geolocationRequest($request)) {
            $this->redirectAllowed = true;
            $url = substr($currentUrl, strlen($baseUrl) - 1);
            return $url;
        }

        switch ($applyLogic) {
            case Logic::ALL_URLS :
                $this->redirectAllowed = true;
                $url = substr($currentUrl, strlen($baseUrl) - 1);
                return $url;
                break;
            case Logic::SPECIFIED_URLS :
                $acceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_urls'));
                foreach ($acceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        $this->redirectAllowed = true;
                        return $url;
                    }
                }
                break;
            case Logic::EXCEPT_URLS :
                $exceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/excepted_urls'));
                foreach ($exceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        $this->redirectAllowed = false;
                        return $url;
                    } else {
                        $this->redirectAllowed = true;
                    }
                }
                break;
            case Logic::HOMEPAGE_ONLY :
                $routeName = $request->getRouteName();
                $action = $request->getActionName();
                if ($routeName == 'cms' && $action == 'index') {
                    $this->redirectAllowed = true;
                }
                break;
            case Logic::SPECIFIED_PATH :
                $acceptedPath = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_path'));
                $currentPath = parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH);

                if ($logEnabled) {
                    $this->_logger->info("Current path " . $currentPath);
                }

                foreach ($acceptedPath as $path) {
                    $path = trim($path);
                    if ($currentPath && $path && $path == $currentPath) {
                        $this->redirectAllowed = true;
                        return $currentPath;
                    }
                }
                break;
        }
        return '';
    }

    protected function geolocationRequest($request) {
        return (null !== $request->getParam("geolocation") && $request->getParam("geolocation") == 1);
    }

}
